import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
SECRET_KEY = "Shuuut... It's a secret"

DEBUG = False
ALLOWED_HOSTS = []
INSTALLED_APPS = [
    'django.contrib.contenttypes',
    'pony_unitfield.tests.testproject',
]

MIDDLEWARE = []

# ROOT_URLCONF = 'pony_unitfield.tests.testproject.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [],
        },
    },
]

# WSGI_APPLICATION = 'pony_unitfield.tests.testproject.wsgi.application'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

AUTH_PASSWORD_VALIDATORS = []

LANGUAGE_CODE = 'en-us'
TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True
STATIC_URL = '/static/'



from django.db import models
from pony_unitfield.fields import UnitField


class Obj(models.Model):
    default = UnitField(unit='foo', null=True)

    no_decimal = UnitField(unit='bar', decimal=0, null=True)
    one_decimal = UnitField(unit='ham', decimal=1, null=True)

    non_spaced = UnitField(unit='bone', spaced_display=False, null=True)



try:
    from functools import partialmethod
except ImportError:
    from django.utils.functional import curry as partialmethod

from django.db.models import fields


def _get_FIELD_display(self, field):
    value = getattr(self, field.attname)
    if value is None:
        return
    template = ''
    template += '{:d}' if field.decimal == 0 else '{:.%sf}' % field.decimal
    template += ' ' if field.spaced_display else ''
    template += '{!s:s}'
    return template.format(value, field.unit)


class UnitField(fields.FloatField):
    def __init__(self, unit, decimal=3, spaced_display=True, *args, **kwargs):
        super(UnitField, self).__init__(*args, **kwargs)
        self.unit = unit
        self.decimal = decimal
        self.spaced_display = spaced_display

    def deconstruct(self):
        name, path, args, kwargs = super(UnitField, self).deconstruct()
        args.extend(('unit',))
        kwargs.update(decimal=3, spaced_display=True)
        return name, path, args, kwargs

    def contribute_to_class(self, cls, name, private_only=False):
        super(UnitField, self).contribute_to_class(cls, name, private_only)
        setattr(cls, 'get_%s_display' % self.name,
                partialmethod(_get_FIELD_display, field=self))



from django.test import TestCase
from pony_unitfield.tests.testproject import models


class GetDisplayTestCase(TestCase):
    def test_default(self):
        obj = models.Obj(default=42)
        display = obj.get_default_display()
        self.assertEqual(display, '42.000 foo')

    def test_no_decimal(self):
        obj = models.Obj(no_decimal=42)
        display = obj.get_no_decimal_display()
        self.assertEqual(display, '42 bar')

    def test_one_decimal(self):
        obj = models.Obj(one_decimal=42)
        display = obj.get_one_decimal_display()
        self.assertEqual(display, '42.0 ham')

    def test_non_spaced(self):
        obj = models.Obj(non_spaced=42)
        display = obj.get_non_spaced_display()
        self.assertEqual(display, '42.000bone')



#!/usr/bin/env python
from setuptools import setup, find_packages
import pony_unitfield


def read_file(name):
    with open(name) as fd:
        return fd.read()

keywords = ['django', 'models', 'fields', 'unit', 'decimal', 'float']

setup(
    name='pony-unitfield',
    version=pony_unitfield.__version__,
    description=pony_unitfield.__doc__,
    long_description=read_file('README.rst'),
    author=pony_unitfield.__author__,
    author_email=pony_unitfield.__email__,
    # install_requires=read_file('requirements.txt'),
    license='BSD',
    url=pony_unitfield.__url__,
    keywords=keywords,
    packages=find_packages(exclude=[]),
    include_package_data=True,
    test_suite='runtests.main',
    tests_require=read_file('requirements-tests.txt'),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
